# -*- coding: utf-8 -*-
import scrapy, re, json
from datetime import datetime, date
from prensaHn.items import NoticiasItem

"""
MEDIO:
La Prensa, Honduras
USO:
// Si se quiere obtener todas las noticias desde las más actuales hasta las más antiguas. //
scrapy crawl noticias --nolog -s filename=noticias.json
-------------------------------------------------------------------------------------------------
// Si se quiere obtener todas las noticias desde las más actuales hasta una fecha específica. //
scrapy crawl noticias --nolog -s filename=noticias.json -a year=2018 -a month=3 -a day=7
-------------------------------------------------------------------------------------------------
Después será necesario hacer uso del archivo parse_date_files.py para que las noticias contenidas
en noticias.json sean separadas en archivos por fecha.
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class ImportantData(scrapy.Item):
    CONTINUE_SEARCHING = scrapy.Field()
    LAST_LINK = scrapy.Field()
    page = scrapy.Field()
    section_url = scrapy.Field()
    url = scrapy.Field()


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        if year is not None and month is not None and day is not None:
            self.stopDate = date(int(year), int(month), int(day))
        else:
            self.stopDate = None

        baseURL = "http://www.laprensa.hn/"

        sectionList = ["economia", "mundo", "tecnologia", "cine", "cultura", "turismo",
                       "honduras", "sucesos", "espectaculos", "deportes"]
        # sectionList = ["economia"]

        if self.stopDate is None:
            for s in sectionList:
                info = ImportantData()
                info['page'] = 1
                request = scrapy.Request(url=baseURL + s, callback=self.parse)
                request.meta['item'] = info
                yield request

        else:
            for s in sectionList:
                info = ImportantData()
                info['page'] = 1
                info['CONTINUE_SEARCHING'] = False
                request = scrapy.Request(url=baseURL + s, callback=self.parse_with_stop_date)
                request.meta['item'] = info
                yield request


    def parse(self, response):
        searchData = response.meta['item']
        CONTINUE_SEARCHING = True
        if searchData['page'] == 1:
            searchData['section_url'] = response.url + "/"
            entrySet = set(response.css('article.entry').css('div.content').css('a::attr(href)').extract())
            entrySet.remove(searchData['section_url'])
            linkSet = set(response.css('article.grid').css('div.content').css('a::attr(href)').extract())
            linkSet.remove(searchData['section_url'])
            linkSet.union(entrySet)

        else:
            linkSet = set(response.css('article.grid').css('div.content').css('a::attr(href)').extract())
            try:
                linkSet.remove(searchData['section_url'])
            except KeyError:
                CONTINUE_SEARCHING = False

        for link in linkSet:
            yield scrapy.Request(url=link, callback=self.parse_item)

        if CONTINUE_SEARCHING:
            searchData['page'] += 1
            page = searchData['page']
            url = searchData['section_url']
            request = scrapy.Request(url=url + "?page=" + str(page), callback=self.parse)
            request.meta['item'] = searchData
            yield request


    def parse_with_stop_date(self, response):
        searchData = response.meta['item']
        CONTINUE_SEARCHING = searchData['CONTINUE_SEARCHING']

        if not CONTINUE_SEARCHING:
            if searchData['page'] == 1:
                searchData['section_url'] = response.url + "/"
                entrySet = set(response.css('article.entry').css('div.content').css('a::attr(href)').extract())
                entrySet.remove(searchData['section_url'])
                linkSet = set(response.css('article.grid').css('div.content').css('a::attr(href)').extract())
                linkSet.remove(searchData['section_url'])
                linkSet.union(entrySet)
                linkList = list(linkSet)

            else:
                linkSet = set(response.css('article.grid').css('div.content').css('a::attr(href)').extract())
                try:
                    linkSet.remove(searchData['section_url'])
                    linkList = list(linkSet)
                except KeyError:
                    linkList = []

            for link in linkList:
                info = ImportantData()
                info['url'] = response.url
                info['page'] = searchData['page']
                info['section_url'] = searchData['section_url']
                if link == linkList[-1]: info['LAST_LINK'] = True
                else: info['LAST_LINK'] = False
                reqst = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                reqst.meta['item'] = info
                yield reqst

        else:
            searchData['CONTINUE_SEARCHING'] = False
            searchData['page'] += 1
            page = searchData['page']
            url = searchData['section_url']
            request = scrapy.Request(url=url + "?page=" + str(page), callback=self.parse_with_stop_date)
            request.meta['item'] = searchData
            yield request


    def parse_item(self, response):
        item = NoticiasItem()
        res = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
        newsData = json.loads(res)

        item['date'] = newsData['datePublished'][:-1]
        item['title'] = newsData['headline']

        try:
            topic = newsData['articleSection']
        except:
            topic = None
        item['topic'] = topic

        item['text'] = newsData['articleBody']

        item['url'] = response.url

        yield item


    def parse_item_with_stop_date(self, response):
        res = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
        newsData = json.loads(res)
        d = newsData['datePublished']
        d = d[:d.find("T")]
        dt = datetime.strptime(d, '%Y-%m-%d').date()

        if dt >= self.stopDate:
            info = response.meta['item']
            item = NoticiasItem()
            text = ''

            item['date'] = newsData['datePublished'][:-1]
            item['title'] = newsData['headline']

            try:
                topic = newsData['articleSection']
            except:
                topic = None
            item['topic'] = topic

            item['text'] = newsData['articleBody']

            item['url'] = response.url

            yield item

            if info['LAST_LINK']:
                info['CONTINUE_SEARCHING'] = True
                request = scrapy.Request(url=info['url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = info
                yield request
